;Filename: TVal.s
;Part: ADuC703x.
;Created: 12/04/2006
;Author: Eckart Hartmann of Analog Devices
;This file is in beta sampling.

;This software is supplied subject our standard software license agreement.
;A non-binding summary is as follows:
;"You may use the software in your own code at your own risk.
;" ADI inc disclaims any warranty or liability of its use".
;The full standard software license agreement is available on request and
; remains the binding agreement.

#include        "..\Fla\ADuC703xi.equ"
Z_TnLD      DEFINE  0x300
Z_TnVAL     DEFINE  0x304
Z_TnCON     DEFINE  0x308
Z_TnCLRI    DEFINE  0x30c
Z_TnCAP     DEFINE  0x310

; TVal==========Writes/reads timer T value and starts/stops timer.
; C Function prototype: int TVal(int iT, int iTLd, int iTGo);
;iT:{T_T0,T_T1,T_T2,T_T3,T_T4}
;iTLd:{0-0xffffffff} Reload value - 16 bit only for T0, T3 and T4.
;iTGo:{T_STOP,T_RUN,T_RD}
; Description of Function: Sets the reload value of iT and starts/stops iT.
; User interface:
;  Set iT to desired Timer, 0 to 4
;   Set iTGo to one of the following values:
;       0 = T_STOP -> Stop timer iT.
;       1 = T_RUN  -> write TnLD and Run timer iT.
;       2 = T_RD   -> only read TnVAL.
;   Call TVal(iT, iTLd, iTGo).
;       Depending on iTGo Stop or run timer iT with
;       iTLd as start value or only read TnVAL.
;   Returns current TnVAL in  r0.
; Robustness:
;   Interupt functionality may be added separately.
; Side effects: Corrupts r1 and r3.

                RSEG CODE
                MODULE TVal
                PUBLIC TVal
                CODE32

TVal:   mov     r3,#0xff0000          ;r3 = Z_BASE
        orr     r3,r3,#0xff000000
        cmp     r0,#4                 ;Ensure valid timer.
        andne   r0,r0,#3
        orr     r3,r3,r0,lsl#5        ;Z_BASE |= r0<<5;

        cmp     r2,#1                 ;if(iTGo>1)
        bhi     TRVal                 ; only read TVAL.
        cmp     r0,#0                 ;If T0
        addeq   r3,r3,#4              ; Z_BASE += 4.
        cmp     r2,#1                 ;if(iTGo==1)
        ldr     r2,[r3,#Z_TnCON]      ; r2 = TnCON;
        orreq   r2,r2,#0x80           ; set enable bit.
        bicmi   r2,r2,#0x80           ;if(iTGo<1) clear enable bit.
        str     r2,[r3,#Z_TnCON]      ;TnCON = r2;
        bic     r3,r3,#4              ;Restore Z_BASE.
        streq   r1,[r3,#Z_TnLD]       ;if(iTGo==1) TnLD = iTLd;
TRVal:  ldr     r0,[r3,#Z_TnVAL]      ;return TnVAL;
        bx      lr

                 ENDMOD  TVal
                 END

